﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using GestioDespeses;

namespace Dades
{
    public class DAArxiu
    {
        //Retorna el id més alt i suma 1
        public static int idArxiu()
        {
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from arx in bd.DBARXIUSOLICITUD
                      select (int?)arx.IDARXIU).Max() ?? 0;
            int _id = id + 1;
            return _id;
        }

        //Afegeix arxiu a la base de dades
        public void AfegirArxiu(int solicitud, string nomArxiu, int length, byte[] _arxiu)
        {
            try
            {
                SolicitudsEntities bd = new SolicitudsEntities(ConfigurationManager.ConnectionStrings["SolicitudsEntities"].ToString());
                DBARXIUSOLICITUD files = new DBARXIUSOLICITUD();

                bd.DBARXIUSOLICITUD.AddObject(new DBARXIUSOLICITUD
                {
                    IDARXIU = idArxiu(),
                    IDSOLICITUD = solicitud,
                    ARXNOM = new FileInfo(nomArxiu).Name,
                    ARXMIDA = length,
                    ARXIU = _arxiu
                });
                bd.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Actualitza arxiu a la base de dades
        public void ActualitzarArxiu(int solicitud, string nomArxiu, int length, byte[] _arxiu)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBARXIUSOLICITUD files = (from arx in bd.DBARXIUSOLICITUD
                                              where arx.IDSOLICITUD == solicitud
                                              select arx).FirstOrDefault();

                    files.IDSOLICITUD = solicitud;
                    files.ARXNOM = new FileInfo(nomArxiu).Name;
                    files.ARXMIDA = length;
                    files.ARXIU = _arxiu;

                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Suprimeix arxiu
        public void DeleteById(int Id)
        {
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {
                DBARXIUSOLICITUD resultat = (from p in bd.DBARXIUSOLICITUD
                                             where p.IDSOLICITUD == Id
                                             select p).FirstOrDefault();

                if (resultat != null)
                {
                    bd.DBARXIUSOLICITUD.DeleteObject(resultat);
                    bd.SaveChanges();
                }
            }
        }

        //llista les dades de l'arxiu de la sol·licitud cercada
        public List<DBARXIUSOLICITUD> GetByArxiu(int idSolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBARXIUSOLICITUD> resultat = (from tip in bd.DBARXIUSOLICITUD
                                                             join sol in bd.DBSOLICITUD
                                                             on tip.IDSOLICITUD equals sol.IDSOLICITUD
                                                             where tip.IDSOLICITUD == idSolicitud
                                                             select tip);


                    return DadesDinamic.Llista<DBARXIUSOLICITUD>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Comprova si existeix un arxiu segons id de la Sol·licitud
        public bool ExisteixArxiu(int idSolicitud)
        {
            int idSol = 0;
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {
                var existeix = (from sol in bd.DBARXIUSOLICITUD
                                where sol.IDSOLICITUD == idSolicitud
                                select sol);
                foreach (var row in existeix)
                {
                    idSol = row.IDSOLICITUD;
                }
                if (idSol > 0)
                    return true;
                else
                    return false;
            }
        }
    }
}







