﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestioDespeses;

namespace Dades
{
    public class DACalendari
    {
        //Retorna el id Calendari de la data del festiu introduida
        public int GetIdCalendari(DateTime data)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBCALENDARI> resultat = (from cal in bd.DBCALENDARI
                                                        where cal.CALDIESFESTIUS == data
                                                        select cal);

                    foreach (var row in resultat)
                    {
                        id = row.IDCALENDARI;
                    }
                    if (id == 0)
                    {
                        id = 4003;
                    }
                    return id;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el descripcio Calendari de la data del festiu introduida
        public string GetDescripcioCalendari(DateTime data)
        {
            try
            {
                string descripcio = String.Empty; 
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBCALENDARI> resultat = (from cal in bd.DBCALENDARI
                                                        where cal.CALDIESFESTIUS == data
                                                        select cal);

                    foreach (var row in resultat)
                    {
                        descripcio = row.CALDESCRIPCIO;
                    }

                    return descripcio;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //llista tots els festius
        public List<DBCALENDARI> GetFestius()
        {
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {

                var resultat = (from cal in bd.DBCALENDARI
                                where cal.CALDIESFESTIUS != null
                                select cal);

                return resultat.ToList();
            }
        }

        //Comprova si la data del festiu introduït existeix
        public int IsFestiu(DateTime festiu)
        {
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {
                int id = 0;
                var resultat = (from cal in bd.DBCALENDARI
                                where cal.CALDIESFESTIUS == festiu
                                select cal);

                foreach (var row in resultat)
                {
                    id = row.IDCALENDARI;
                }
                return id;
            }
        }

        //Afegeix el nou festiu a la base de dades
        public void AfegirFestiu(DateTime data, string descripcio)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    bd.DBCALENDARI.AddObject(new DBCALENDARI
                    {
                        IDCALENDARI = idCalendari(),
                        CALDIESFESTIUS = data,
                        CALDESCRIPCIO = descripcio
                    });
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna id més alt del Calendari i suma 1
        public int idCalendari()
        {
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from idCal in bd.DBCALENDARI
                      select (int?)idCal.IDCALENDARI).Max() ?? 0;
            int _id = id + 1;
            return _id;
        }

        //Elimina el festiu introduït de la base de dades
        public void EliminarFestiu(int idFestiu)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBCALENDARI festiu = (from cal in bd.DBCALENDARI
                                          where cal.IDCALENDARI == idFestiu
                                          select cal).FirstOrDefault();

                    if (festiu != null)
                    {
                        bd.DBCALENDARI.DeleteObject(festiu);
                        bd.SaveChanges();
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
    }
}
