﻿using System;
using System.Linq;
using GestioDespeses;

namespace Dades
{
    public class DADepartament
    {
        //Retorna el id del departament segons la descripció indicada
        public int GetDepartament(string departament)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBDEPARTAMENT> depart = (from p in bd.DBDEPARTAMENT
                                                        where p.DEPDESCRIPCIO == departament
                                                        select p);
                    foreach (var row in depart)
                    {
                        id = row.IDDEPARTAMENT;
                    }
                    return id;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Retorna la descripció del departament segons el id del departament indicat
        public string GetDescripcioDepartament(int departament)
        {
            try
            {
                string descripcio = String.Empty;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBDEPARTAMENT> depart = (from p in bd.DBDEPARTAMENT
                                                        where p.IDDEPARTAMENT == departament
                                                        select p);

                    foreach (var row in depart)
                    {
                        descripcio = row.DEPDESCRIPCIO;
                    }
                    return descripcio;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Elimina el departament segons la descripció indicada
        public void Eliminar(string departament)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBDEPARTAMENT depart = (from p in bd.DBDEPARTAMENT
                                            where p.DEPDESCRIPCIO == departament
                                            select p).FirstOrDefault();

                    if (depart != null)
                    {
                        bd.DBDEPARTAMENT.DeleteObject(depart);
                        bd.SaveChanges();
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Dona d'alta un nou departament
        public void Alta(string descripcio)
        {
            try
            {
                int id = idDept();
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {

                    bd.DBDEPARTAMENT.AddObject(new DBDEPARTAMENT
                    {
                        IDDEPARTAMENT = id,
                        DEPDESCRIPCIO = descripcio,
                        DEPDATAALTA = System.DateTime.Now
                    });
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el id més alt i suma 1
        public int idDept()
        {
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from idDept in bd.DBDEPARTAMENT
                      select (int?)idDept.IDDEPARTAMENT).Max() ?? 0;
            int _id = id + 1;
            return _id;
        }
    }
}
