﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestioDespeses;

namespace Dades
{
    public class DAHistoric
    {
        //Obtenim id sol·licitud tancada segons la seva tipologia   
        public int GetSolicituds(string tipologia)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                        join tip in bd.DBTIPOLOGIA
                                                        on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                                        where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                                        select sol);

                    foreach (var row in resultat)
                    {
                        id = row.IDSOLICITUD;
                    }
                    return id;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Llista les dades de la sol·licitud indicada i que estiquin en estat tancat
        public  List<DBSOLICITUD> GetSolicitudsHistoric(int idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                        join tip in bd.DBTIPOLOGIA
                                                        on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                                        where sol.IDSOLICITUD == idsolicitud &&
                                                        sol.SOLESTAT == "Tancada"
                                                        orderby sol.IDSOLICITUD
                                                        select sol);

                    return DadesDinamic.Llista<DBSOLICITUD>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el número total de sol·licituds tancades segons les dades d'entrada
        public int GetHistoricQuantitat(string tipologia, DateTime? dataAlta, DateTime? dataFi)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    int num = 0;
                    if (tipologia != String.Empty && dataAlta == null && dataFi == null)
                    {
                        var numero = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBTIPOLOGIA
                                      on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                      where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }
                    if (tipologia == String.Empty && dataAlta != null && dataFi != null)
                    {
                        var numero = (from sol in bd.DBSOLICITUD
                                      where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      && sol.SOLESTAT == "Tancada"
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }

                    if (tipologia != String.Empty && dataAlta != null && dataFi != null)
                    {

                        var numero = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBTIPOLOGIA
                                      on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                      where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                      && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }
                    return num;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Retorna l'import total de totes les sol·licituds tancades segons les dades indicades
        public float GetHistoricImport(string tipologia, DateTime? dataAlta, DateTime? dataFi)
        {
            DATipologia tipus = new DATipologia();
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    float num = 0;

                    if (tipologia != String.Empty && dataAlta == null && dataFi == null)
                    {
                        var _tipus = tipus.GetIdTipologia(tipologia);

                        var import = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBTIPOLOGIA
                                      on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                      where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                      select sol.SOLIMPORT).Sum();

                        num = import;
                    }
                    if (tipologia == String.Empty && dataAlta != null && dataFi != null)
                    {
                        var import = (from sol in bd.DBSOLICITUD
                                      where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      && sol.SOLESTAT == "Tancada"
                                      select sol.SOLIMPORT).Sum();
                        num = import;
                    }

                    if (tipologia != String.Empty && dataAlta != null && dataFi != null)
                    {
                        var import = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBTIPOLOGIA
                                      on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                      where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                      && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      select sol.SOLIMPORT).Sum();

                        num = import;
                    }
                    return num;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Llista totes les sol·licituds tancades segons les dades d'entrada
        public List<DBSOLICITUD> GetHistoricLlista(string tipologia, DateTime? dataAlta, DateTime? dataFi)
        {
            try
            {
                IQueryable<DBSOLICITUD> llista = null;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    if (tipologia != String.Empty && dataAlta == null && dataFi == null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            join tip in bd.DBTIPOLOGIA
                                                            on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                                            where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                                            orderby sol.IDSOLICITUD
                                                            select sol);

                        llista = resultat;
                    }
                    if (tipologia == String.Empty && dataAlta != null && dataFi != null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                                            && sol.SOLESTAT == "Tancada"
                                                            orderby sol.IDSOLICITUD
                                                            select sol);
                        llista = resultat;
                    }

                    if (tipologia != String.Empty && dataAlta != null && dataFi != null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            join tip in bd.DBTIPOLOGIA
                                                            on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                                            where tip.TIPDESCRIPCIO == tipologia && sol.SOLESTAT == "Tancada"
                                                            && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                                            orderby sol.IDSOLICITUD
                                                            select sol);

                        llista = resultat;
                    }
                    return DadesDinamic.Llista<DBSOLICITUD>(llista);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el número total sol·licituds tancades consultades per l'administrador segons dades d'entrada indicades
        public int GetHistoricQuantitatAdmin(string departament, DateTime? dataAlta, DateTime? dataFi)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    int num = 0;
                    if (departament != String.Empty && dataAlta == null && dataFi == null)
                    {
                        var numero = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBUSUARI
                                      on sol.IDUSUARI equals tip.IDUSUARI
                                      join dpt in bd.DBDEPARTAMENT
                                      on tip.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                      where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }
                    if (departament == String.Empty && dataAlta != null && dataFi != null)
                    {
                        var numero = (from sol in bd.DBSOLICITUD
                                      where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      && sol.SOLESTAT == "Tancada"
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }

                    if (departament != String.Empty && dataAlta != null && dataFi != null)
                    {

                        var numero = (from sol in bd.DBSOLICITUD
                                      join tip in bd.DBUSUARI
                                      on sol.IDUSUARI equals tip.IDUSUARI
                                      join dpt in bd.DBDEPARTAMENT
                                      on tip.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                      where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                      && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      select new { sol.IDSOLICITUD }).Count();

                        num = numero;
                    }
                    return num;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Retorna import total de sol·licituds tancades consultades per l'administrador segons dades d'entrada indicades
        public float GetHistoricImportAdmin(string departament, DateTime? dataAlta, DateTime? dataFi)
        {
            DATipologia tipus = new DATipologia();
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    float num = 0;
                    if (departament != String.Empty && dataAlta == null && dataFi == null)
                    {
                        var import = (from sol in bd.DBSOLICITUD
                                      join usu in bd.DBUSUARI
                                      on sol.IDUSUARI equals usu.IDUSUARI
                                      join dpt in bd.DBDEPARTAMENT
                                      on usu.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                      where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                      select sol.SOLIMPORT).Sum();

                        num = import;
                    }
                    if (departament == String.Empty && dataAlta != null && dataFi != null)
                    {
                        var import = (from sol in bd.DBSOLICITUD
                                      where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      && sol.SOLESTAT == "Tancada"
                                      select sol.SOLIMPORT).Sum();
                        num = import;
                    }

                    if (departament != String.Empty && dataAlta != null && dataFi != null)
                    {
                        var import = (from sol in bd.DBSOLICITUD
                                      join usu in bd.DBUSUARI
                                      on sol.IDUSUARI equals usu.IDUSUARI
                                      join dpt in bd.DBDEPARTAMENT
                                      on usu.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                      where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                      && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                      select sol.SOLIMPORT).Sum();
                        num = import;
                    }
                    return num;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Llista les dades de les sol·licituds tancades consultades per l'administrador segons dades d'entrada indicades
        public List<DBSOLICITUD> GetHistoricLlistaAdmin(string departament, DateTime? dataAlta, DateTime? dataFi)
        {
            try
            {
                IQueryable<DBSOLICITUD> llista = null;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    if (departament != String.Empty && dataAlta == null && dataFi == null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            join usu in bd.DBUSUARI
                                                            on sol.IDUSUARI equals usu.IDUSUARI
                                                            join dpt in bd.DBDEPARTAMENT
                                                            on usu.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                                            where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                                            orderby sol.IDSOLICITUD
                                                            select sol);
                        llista = resultat;
                    }
                    if (departament == String.Empty && dataAlta != null && dataFi != null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            where sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                                            && sol.SOLESTAT == "Tancada"
                                                            orderby sol.IDSOLICITUD
                                                            select sol);
                        llista = resultat;
                    }

                    if (departament != String.Empty && dataAlta != null && dataFi != null)
                    {
                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            join usu in bd.DBUSUARI
                                                            on sol.IDUSUARI equals usu.IDUSUARI
                                                            join dpt in bd.DBDEPARTAMENT
                                                            on usu.IDDEPARTAMENT equals dpt.IDDEPARTAMENT
                                                            where dpt.DEPDESCRIPCIO == departament && sol.SOLESTAT == "Tancada"
                                                            && sol.SOLDATAALTA >= dataAlta && sol.SOLDATAFI <= dataFi
                                                            orderby sol.IDSOLICITUD
                                                            select sol);
                        llista = resultat;
                    }

                    return DadesDinamic.Llista<DBSOLICITUD>(llista);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
    }
}
