﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Mail;
using Entities;
using GestioDespeses;

namespace Dades
{
    public class DAIncidencies
    {
        //Llista totes les incidències en estat obert
        public List<DBINCIDENCIES> GetAll()
        {
            try
            {
                List<DBINCIDENCIES> llista = new List<DBINCIDENCIES>();
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBINCIDENCIES> resultat = (from inc in bd.DBINCIDENCIES
                                                          where inc.INCESTAT == "Oberta"
                                                          select inc);

                    return DadesDinamic.Llista<DBINCIDENCIES>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna les dades de la incidència del id indicat
        public DBINCIDENCIES GetIncidenciesId(int incidencia)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    return bd.DBINCIDENCIES.FirstOrDefault(x => x.IDINCIDENCIA == incidencia);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Alta nova incidència 
        public void AfegirIncidencia(clsIncidencies inc)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    bd.DBINCIDENCIES.AddObject(new DBINCIDENCIES
                     {
                         IDINCIDENCIA = idInci(),
                         IDUSUARI = inc.idUsuari,
                         INCDESCRIPCIO = inc.descripcio,
                         INCESTAT = "Oberta",
                         INCALTA = System.DateTime.Now,
                         INCBREUDESCRIPCIO = inc.breudescripcio
                     });
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Actualitza incidència quan aquesta és resposta. Canvia l'estat a Tancat. Envia email a l'usuari amb la resposta
        public void ActualitzarIncidencia(clsIncidencies inc)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBINCIDENCIES resultat = (from inci in bd.DBINCIDENCIES
                                              where inci.IDINCIDENCIA == inc.idIncidencia
                                              select inci).FirstOrDefault();

                    resultat.IDINCIDENCIA = inc.idIncidencia;
                    resultat.IDUSUARI = inc.idUsuari;
                    resultat.INCDESCRIPCIO = inc.descripcio;
                    resultat.INCESTAT = "Tancada";
                    resultat.INCALTA = inc.dataAlta;
                    resultat.INCTANCAT = System.DateTime.Now;
                    resultat.INCRESPOSTA = inc.resposta;
                    resultat.INCBREUDESCRIPCIO = inc.breudescripcio;
                    //  enviarMail(inc);    
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Enviar email a l'usuari amb la resposta a la incidència 
        private void enviarMail(clsIncidencies inc)
        {
            MailMessage mail = new MailMessage();
            mail.Subject = String.Format("Resposta a la inciència" + inc.idIncidencia);
            DAUsuari usuari = new DAUsuari();
            mail.To.Add(new MailAddress(usuari.GetMail(inc.idUsuari)));
            mail.From = new MailAddress("quique7202@hotmail.com", "Nom Empresa");
            mail.Body = string.Format(inc.nom);
            mail.Body += Environment.NewLine;
            mail.Body = string.Format(inc.breudescripcio);
            mail.Body += Environment.NewLine;
            mail.Body = string.Format(inc.descripcio);
            mail.Body += Environment.NewLine;
            mail.Body = string.Format(inc.resposta);
            mail.Body += Environment.NewLine;
            mail.Body += (string.Format("Company corp."));
            SmtpClient server = new SmtpClient("smtp.mail.hotmail.com", 25);
            server.Credentials = new NetworkCredential("pcsol", "welcome");
            server.EnableSsl = false;
            server.Send(mail);
        }

        //Retorna id màxim i suma 1
        public int idInci()
        {
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from inci in bd.DBINCIDENCIES
                      select (int?)inci.IDINCIDENCIA).Max() ?? 0;
            int _id = id + 1;
            return _id;
        }
    }
}
