﻿using System;
using System.Collections.Generic;
using System.Linq;
using Entities;
using GestioDespeses;


namespace Dades
{
    public class DASolicitud
    {
        //Llista totes les sol·licituds
        public List<DBSOLICITUD> Dades()
        {
            SolicitudsEntities context = new SolicitudsEntities();

            return context.DBSOLICITUD.ToList();
        }

        //Afegeix una nova sol·licitud a la base de dades
        public int AfegirSolicitud(Solicitud sol, string nom, string email)
        {
            try
            {
                int id = idSol();
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    bd.DBSOLICITUD.AddObject(new DBSOLICITUD
                    {
                        IDSOLICITUD = id,
                        IDCALENDARI = sol.idCalendari,
                        IDUSUARI = sol.idUsuari,
                        IDTIPOLOGIA = sol.idTipologia,
                        SOLDESCRIPCIO = sol.descripcio,
                        SOLDATAALTA = DateTime.Now.Date,
                        SOLESTAT = "Oberta",
                        SOLIMPORT = sol.import,
                        SOLACTIVA = "S",
                        SOLDATADESPESA = sol.dataDespesa,
                        SOLARXIU = sol.arxiu
                    });
                    bd.SaveChanges();
                }
                return id;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el id màxim i suma 1
        public int idSol()
        {
            int _id = 0;
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from idSol in bd.DBSOLICITUD
                      select (int?)idSol.IDSOLICITUD).Max() ?? 0;
            
            if (id == 0)
                _id = 1000;
            else
               _id = id + 1;

            return _id;
        }

        //Elimina sol·licitud
        public void EliminarSolicitud(int idSolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBSOLICITUD eliminar = (from sol in bd.DBSOLICITUD
                                            where sol.IDSOLICITUD == idSolicitud
                                            select sol).FirstOrDefault();
                    eliminar.SOLESTAT = "Anulada";
                    eliminar.SOLACTIVA = "N";
                    eliminar.SOLMODIFICACIO = System.DateTime.Now;

                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Llista les sol·licituds per editar
        public List<DBSOLICITUD> GetSolicitudsEditar(int idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                        join tip in bd.DBTIPOLOGIA
                                                        on sol.IDTIPOLOGIA equals tip.IDTIPOLOGIA
                                                        where sol.IDSOLICITUD == idsolicitud
                                                        orderby sol.IDSOLICITUD
                                                        select sol);

                    return DadesDinamic.Llista<DBSOLICITUD>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        //Llista les sol·licituds consultades segons parametres d'entrada
        public List<DBSOLICITUD> GetAllSolicituds(int usuari, string _solicitud, DateTime? data, string _estat)
        {
            IQueryable<DBSOLICITUD> llista = null;
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {               
                if (_solicitud != String.Empty && data == null && _estat == String.Empty)
                {
                    int solicitud = Convert.ToInt32(_solicitud);
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari && sol.IDSOLICITUD == solicitud
                                   select sol);
                    llista = resultat;
                }
                if (_solicitud == String.Empty && data != null && _estat != String.Empty)
                {
                    var resultat = (from sol in bd.DBSOLICITUD
                                  where sol.IDUSUARI == usuari && sol.SOLDATAALTA == data && sol.SOLESTAT == _estat
                                  select sol);

                    llista = resultat;
                }
                if (_solicitud == String.Empty && data == null && _estat != String.Empty)
                {
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari && sol.SOLESTAT == _estat
                                    select sol);

                    llista = resultat;
                }
                if (_solicitud != String.Empty && data != null && _estat != String.Empty)
                {
                    int solicitud = Convert.ToInt32(_solicitud);
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari && sol.SOLDATAALTA == data && sol.SOLESTAT == _estat
                                  && sol.IDSOLICITUD == solicitud
                                  select sol);

                    llista = resultat;
                }
                if (_solicitud == String.Empty && data != null && _estat == String.Empty)
                {
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari && sol.SOLDATAALTA == data 
                                    select sol);

                    llista = resultat;
                }
                if (_solicitud != String.Empty && data == null && _estat != String.Empty)
                {
                    int solicitud = Convert.ToInt32(_solicitud);
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari && sol.SOLESTAT == _estat
                                    && sol.IDSOLICITUD == solicitud
                                    select sol);

                    llista = resultat;
                }               
                return DadesDinamic.Llista<DBSOLICITUD>(llista);
            }
        }

        //Llista les sol·licituds per consultar
        public List<DBSOLICITUD> GetAllSolicitudsSenseFiltre(int usuari)
        {            
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {                
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.IDUSUARI == usuari 
                                    select sol);  

                return DadesDinamic.Llista<DBSOLICITUD>(resultat);
            }
        }

        //Retorna el id de la sol·licitud que no estigui oberta o rebutjada      
        public int GetSolicitudIdNoOberta(int idsolicitud)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                        where sol.IDSOLICITUD == idsolicitud && sol.SOLESTAT != "Oberta" &&
                                                        sol.SOLESTAT != "Rebutjada"
                                                        select sol);

                    foreach (var row in resultat)
                    {
                        id = row.IDSOLICITUD;
                    }
                    return id;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna el id de la sol·licitud que no estigui en estat pendent
        public int GetSolicitudIdPendent(int idsolicitud)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                        where sol.IDSOLICITUD == idsolicitud && sol.SOLESTAT != "Pendent"
                                                        select sol);

                    foreach (var row in resultat)
                    {
                        id = row.IDSOLICITUD;
                    }
                    return id;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna les dades de la sol·licitud indicada
        public DBSOLICITUD GetSolicitudId(int? idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    return bd.DBSOLICITUD.FirstOrDefault(x => x.IDSOLICITUD == idsolicitud);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Llista les sol·licituds en estat pendent per ser autoritzades
        public List<DBSOLICITUD> GetSolicitudsPendentsAutoritzar()
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    var resultat = (from sol in bd.DBSOLICITUD
                                    where sol.SOLESTAT == "Pendent"
                                    orderby sol.IDSOLICITUD
                                    select sol);

                    return DadesDinamic.Llista<DBSOLICITUD>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna id de la sol·licitud autoritzada i canvia l'estat a Abonament
        public int UpdateEstatSolicitud(int idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBSOLICITUD editar = (from sol in bd.DBSOLICITUD
                                          where sol.IDSOLICITUD == idsolicitud
                                          select sol).FirstOrDefault();
                    editar.SOLESTAT = "Abonament";
                    return bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna id de la sol·licitud enviada a autoritzar. Canvia l'estat a Pendent
        public string UpdateEstatSolicitudAutoritzar(int idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBSOLICITUD editar = (from sol in bd.DBSOLICITUD
                                          where sol.IDSOLICITUD == idsolicitud
                                          select sol).FirstOrDefault();
                    editar.SOLESTAT = "Pendent";
                    editar.SOLNUMREGISTRE = DateTime.Now.ToShortDateString() + editar.IDSOLICITUD;
                    bd.SaveChanges();
                    return editar.SOLNUMREGISTRE;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }       

        //Retorna id sol·licitud rebutjada. Canvia l'estat de Pendent a Rebutjada.
        public int UpdateEstatSolicitudRebutjar(int idsolicitud, string motiu)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBSOLICITUD editar = (from sol in bd.DBSOLICITUD
                                          where sol.IDSOLICITUD == idsolicitud
                                          select sol).FirstOrDefault();
                    editar.SOLESTAT = "Rebutjada";
                    return bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Actualitza sol·licitud editada
        public void UpdateSolicitud(Solicitud soli)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBSOLICITUD rst = (from sol in bd.DBSOLICITUD
                                       where sol.IDSOLICITUD == soli.idSolicitud
                                       select sol).FirstOrDefault();

                    rst.IDTIPOLOGIA = soli.idTipologia;
                    rst.IDCALENDARI = soli.idCalendari;
                    rst.SOLDESCRIPCIO = soli.descripcio;
                    rst.SOLESTAT = "Oberta";
                    rst.SOLIMPORT = soli.import;
                    rst.SOLMODIFICACIO = DateTime.Now;
                    rst.SOLDATADESPESA = soli.dataDespesa;
                    rst.SOLARXIU = soli.arxiu;
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //llista totes les sol·licituds segons paràmetres d'entrada en la cerca pàgina Autoritzar
        public List<DBSOLICITUD> GetSolicitudsAutoritzarConsultar(DateTime? data, string dni, string registre)
        {
            try
            {
                IQueryable<DBSOLICITUD> llista = null;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    if (dni != String.Empty && data == null)
                    {

                        IQueryable<DBSOLICITUD> resultat = (from sol in bd.DBSOLICITUD
                                                            join usu in bd.DBUSUARI
                                                            on sol.IDUSUARI equals usu.IDUSUARI
                                                            where usu.USUDNI == dni
                                                            orderby sol.IDSOLICITUD
                                                            select sol);
                        llista = resultat;
                    }
                    if (dni != String.Empty && data != null)
                    {
                        IQueryable<DBSOLICITUD> resultat2 = (from sol in bd.DBSOLICITUD
                                                             join usu in bd.DBUSUARI
                                                             on sol.IDUSUARI equals usu.IDUSUARI
                                                             where usu.USUDNI == dni && sol.SOLDATADESPESA == data
                                                             orderby sol.IDSOLICITUD
                                                             select sol);
                        llista = resultat2;
                    }

                    if (dni != String.Empty && data != null && registre != String.Empty)
                    {
                        IQueryable<DBSOLICITUD> resultat3 = (from sol in bd.DBSOLICITUD
                                                             join usu in bd.DBUSUARI
                                                             on sol.IDUSUARI equals usu.IDUSUARI
                                                             where usu.USUDNI == dni && sol.SOLDATADESPESA == data
                                                             && sol.SOLNUMREGISTRE == registre
                                                             orderby sol.IDSOLICITUD
                                                             select sol);
                        llista = resultat3;
                    }

                    if (dni != String.Empty && data == null && registre != String.Empty)
                    {
                        IQueryable<DBSOLICITUD> resultat4 = (from sol in bd.DBSOLICITUD
                                                             join usu in bd.DBUSUARI
                                                             on sol.IDUSUARI equals usu.IDUSUARI
                                                             where usu.USUDNI == dni && sol.SOLNUMREGISTRE == registre
                                                             orderby sol.IDSOLICITUD
                                                             select sol);
                        llista = resultat4;
                    }

                    return DadesDinamic.Llista<DBSOLICITUD>(llista);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Comprova si existeix la sol·licitud
        public bool Existeix(int idSolicitud)
        {
            int idSol = 0;
            using (SolicitudsEntities bd = new SolicitudsEntities())
            {
                var existeix = (from sol in bd.DBSOLICITUD
                                where sol.IDSOLICITUD == idSolicitud
                                select sol);
                foreach (var row in existeix)
                {
                    idSol = row.IDSOLICITUD;
                }
                if (idSol > 0)
                    return true;
                else
                    return false;
            }
        }
    }
}
