﻿using System;
using System.Collections.Generic;
using System.Linq;
using Entities;
using GestioDespeses;

namespace Dades
{
    public class DAUsuari
    {
        //Retorna id usuari
        public int GetIdUsuari(string dni)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBUSUARI> resultat = (from usu in bd.DBUSUARI
                                                     where usu.USUDNI == dni
                                                     select usu);
                    foreach (var row in resultat)
                    {
                        id = row.IDUSUARI;
                    }
                    return id;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna id usuari si el dni i la clau introduïdes coincideixen
        public int LoginUsuari(string _usuari, string _clau)
        {
            try
            {
                int id = 0;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBUSUARI> usuari = (from p in bd.DBUSUARI
                                                   where p.USUDNI == _usuari && p.USUCLAU == _clau
                                                   select p);

                    foreach (var row in usuari)
                    {
                        id = row.IDUSUARI;
                    }
                    return id;
                }
            }
            catch (Exception)
            {
                return 0;
            }
        }

        //Llista dades usuari segona la sol·licitud paràmetre d'entrada
        public List<DBUSUARI> GetDniUsuari(int idsolicitud)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBUSUARI> resultat = (from usu in bd.DBUSUARI
                                                     join sol in bd.DBSOLICITUD
                                                     on usu.IDUSUARI equals sol.IDUSUARI
                                                     where sol.IDSOLICITUD == idsolicitud
                                                     select usu);

                    return DadesDinamic.Llista<DBUSUARI>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Llista dades usuari per editar segons dni paràmetre d'entrada
        public List<DBUSUARI> GetEditarUsuari(string usuari)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBUSUARI> resultat = (from usu in bd.DBUSUARI
                                                     where usu.USUDNI == usuari
                                                     select usu);

                    return DadesDinamic.Llista<DBUSUARI>(resultat);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna dades de l'usuari per el id indicat
        public DBUSUARI GetDni(int idusuari)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    return bd.DBUSUARI.FirstOrDefault(x => x.IDUSUARI == idusuari);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna email usuari
        public string GetMail(int idUsuari)
        {
            try
            {
                string mail = String.Empty;
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    IQueryable<DBUSUARI> usuari = (from p in bd.DBUSUARI
                                                   where p.IDUSUARI == idUsuari
                                                   select p);

                    foreach (var row in usuari)
                    {
                        mail = row.USUEMAIL;
                    }
                    return mail;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Alta nou usuari
        public void AfegirUsuari(Usuari usu)
        {
            try
            {
                int id = idUsuari();
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    bd.DBUSUARI.AddObject(new DBUSUARI
                    {
                        IDUSUARI = id,
                        USUDNI = usu.usudni,
                        IDDEPARTAMENT = usu.idDepartament,
                        IDROL = usu.idRol,
                        USUCOGNOM = usu.usucognom,
                        USUEMAIL = usu.email,
                        USUALTA = System.DateTime.Now,
                        USUCLAU = usu.clau,
                        USUPREGUNTA = usu.pregunta,
                        USURESPOSTA = usu.resposta,
                        USUTELEFON = usu.telefon
                    });
                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Retorna id màxim i suma 1
        public int idUsuari()
        {
            SolicitudsEntities bd = new SolicitudsEntities();
            int id = (from idUsu in bd.DBUSUARI
                      select (int?)idUsu.IDUSUARI).Max() ?? 0;
            int _id = id + 1;
            return _id;
        }

        //Actualitza usuari
        public void UpdateUsuari(Usuari usu)
        {
            try
            {
                int id = idUsuari();
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBUSUARI editar = (from usuari in bd.DBUSUARI
                                       where usuari.IDUSUARI == usu.idUsuari
                                       select usuari).FirstOrDefault();

                    editar.USUDNI = usu.usudni;
                    editar.USUCOGNOM = usu.usucognom;
                    editar.USUEMAIL = usu.email;
                    editar.USUTELEFON = usu.telefon;
                    editar.USUCLAU = usu.clau;
                    editar.USUPREGUNTA = usu.pregunta;
                    editar.USURESPOSTA = usu.resposta;
                    editar.IDDEPARTAMENT = usu.idDepartament;
                    editar.IDROL = usu.idRol;

                    bd.SaveChanges();
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        //Elimina usuari
        public void EliminarUsuari(string usuDni)
        {
            try
            {
                using (SolicitudsEntities bd = new SolicitudsEntities())
                {
                    DBUSUARI eliminar = (from usu in bd.DBUSUARI
                                         where usu.USUDNI == usuDni
                                         select usu).FirstOrDefault();

                    if (eliminar != null)
                    {
                        bd.DBUSUARI.DeleteObject(eliminar);
                        bd.SaveChanges();
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
    }
}
