﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data;
using Negoci;
using System.Windows.Forms;
using GestioDespeses.Controls;

namespace GestioDespeses.Administrador
{
    /// <summary>
    /// Gestionar el Calendari dies festius
    /// </summary>
    public partial class GestioCalendari : System.Web.UI.Page
    {
        private List<DBCALENDARI> llista;
        protected void Page_Load(object sender, EventArgs e)
        {
            string usuari = String.Empty;
            if (!this.IsPostBack)
            {             
               if (Session["usuari"] == null)
                  Response.Redirect("~/Account/Login.aspx");

               usuari = Convert.ToString(Session["usuari"]);
                string rolUsuari = BLRol.GetUsuariRol(usuari);
                if(rolUsuari != "Administrador")
                    Response.Redirect("~/Menu.aspx");

                carregarLlistaFestius();
            }
        }

        /// <summary>
        /// Seleccionar dia del calendari per afegir com a festiu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnSeleccionar_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.TextBox1.Text.Length > 0 && (DropDownList1.SelectedValue != String.Empty || txtTipFesta.Text != String.Empty)) //comprovem que s'ha seleccionat una data i el tipus de festiu
                {
                    string dia = DropDownList1.SelectedValue;
                    string tipus = txtTipFesta.Text;
                    DateTime festiu = new DateTime();
                    festiu = DateTime.Parse(Calendar1.SelectedDate.ToString());

                    if (BLCalendari.IsFestiu(festiu) > 0)
                    {//comprovem que la data és nova
                        MessageBox.Show("Aquesta data ja esta marcada com a festiu", "Festiu", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    else
                    {
                        if (tipus != String.Empty) //si s'ha descrit un nou tipus de festa
                        {
                            //Alta del festiu amb la nova descripció
                            AltaFestiu(festiu, tipus);
                        }
                        else
                        {
                            //Alta del festiu amb la selecció
                            AltaFestiu(festiu, dia);
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Seleccioni una data i un tipus de festiu", "Data", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch
            {
                MissatgesError.ComprovarDades();
            }
        }

        /// <summary>
        /// Eliminar dia festiu de la llista
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnTreure_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.TextBox1.Text.Length > 0) //comprovem que s'ha seleccionat una data
                {
                    DateTime festiu = new DateTime();
                    festiu = DateTime.Parse(this.TextBox1.Text);
                    int idFestiu = BLCalendari.IsFestiu(festiu);
                    if (idFestiu <= 0)
                    {//comprovem que la data existeix
                        MessageBox.Show("Aquesta data no esta marcada com a festiu", "Eliminar Festiu", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    else
                    { //Eliminem festiu
                        BLCalendari.EliminarFestiu(idFestiu);
                        MessageBox.Show("Festiu eliminat", "Eliminar Festiu", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        carregarLlistaFestius(); //actualitzem la llista
                    }
                }
                else
                {
                    MessageBox.Show("Seleccioni una data", "Data", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch
            {
                MissatgesError.ComprovarDades();
            }
        }

        /// <summary>
        /// Afegir el dia escollit del calendari al textbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void Calendar1_SelectionChanged(object sender, EventArgs e)
        {
            TextBox1.Text = Convert.ToString(this.Calendar1.SelectedDate.ToShortDateString());
        }

        /// <summary>
        /// Alta dia festiu
        /// </summary>
        /// <param name="festiu">Data del dia festiu.</param>
        /// <param name="descripcio">Descripció dia festiu.</param>
        private void AltaFestiu(DateTime festiu, string descripcio)
        {           
            //fem l'alta de festiu amb la descripció de tipus de festa
            BLCalendari.AfegirFestiu(festiu, descripcio);
            MessageBox.Show("Dia afegit com festiu", "Nou festiu", MessageBoxButtons.OK, MessageBoxIcon.Information);
            carregarLlistaFestius(); //actualitzem la llista
            txtTipFesta.Text = String.Empty;
        }

        /// <summary>
        /// Carregar la llista dels dies festis donats d'alta
        /// </summary>
        private void carregarLlistaFestius()
        {
            int year = DateTime.Now.Year;
            TextBox1.Text = String.Empty;
            int any = 0;
            //Carreguem la llista amb els dies festius
            llista = BLCalendari.GetFestius();
            for (int i = 0; i < llista.Count(); i++)
            {//Carreguem la llista de festius si són de l'any actual o posterior
                DateTime? actYear = llista.ElementAt(i).CALDIESFESTIUS;
                if (actYear != null)
                {
                    any = Convert.ToDateTime(actYear).Year;
                }
                if (any >= year)
                {
                    DataList1.DataSource = llista;
                    DataList1.DataBind();
                }
            }  
        }

        /// <summary>
        /// Tancar i tornar al menú "Configuracions"
        /// </summary>
        protected void btnTancar_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Configuracio.aspx");
        }

        /// <summary>
        /// Netejar i Amagar modal panel
        /// </summary>
        protected void btnCancelar_Click(object sender, EventArgs e)
        {
            txtTipFesta.Text = String.Empty;
            mpeAltaTipusFesta.Hide();
        }

        /// <summary>
        /// Mostrar modal panel
        /// </summary>
        protected void btnAlta_Click(object sender, EventArgs e)
        {
            mpeAltaTipusFesta.Show();
        }
    }
}