﻿using System;
using System.Web.UI.WebControls;
using System.Collections;
using Entities;
using Negoci;
using System.Web.UI;
using GestioDespeses.Controls;

namespace GestioDespeses.Administrador
{
    /// <summary>
    /// Gestionar Usuaris
    /// </summary>
    public partial class GestioUsuari : System.Web.UI.Page
    {
        private string usuDni
        {
            get { return (string)ViewState["usuDni"]; }
            set { ViewState["usuDni"] = value; }
        }

        static ArrayList ar = new ArrayList();

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                string usuari = String.Empty;

                if (Session["usuari"] == null)
                    Response.Redirect("~/Account/Login.aspx");

                usuari = Convert.ToString(Session["usuari"]);
                string rolUsuari = BLRol.GetUsuariRol(usuari);
                if (rolUsuari != "Administrador")
                    Response.Redirect("~/Menu.aspx");    
            }
        }

        /// <summary>
        /// Cercar usuaris segons selecció
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param> 
        protected void btnCercar_Click(object sender, EventArgs e)
        {
            gwConsultar.Visible = true;
        }

        /// <summary>
        /// Tornar a la pàgina Configuració
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnTornar_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Configuracio.aspx");
        }

        /// <summary>
        /// Mostrar modal panel alta nou usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnNou_Click(object sender, EventArgs e)
        {
            mpeAltaUsuari.Show();
        }

        /// <summary>
        ///Netejar dades del filtre
        /// </summary>
        protected void btnNetejar_Click(object sender, System.Web.UI.ImageClickEventArgs e)
        {
            txtDNI.Text = "";
            txtNom.Text = "";
            DropDownList3.SelectedIndex = 0;
        }

        #region [grid]
        /// <summary>
        /// Paginació grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void gwGridConsultar_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                gwGridConsultar.PageIndex = e.NewPageIndex;
                gwGridConsultar.DataBind();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        /// <summary>
        /// Editar/Eliminar Usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        protected void gwGridConsultar_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            try
            {
                //recuperem índex de la selecció
                int intRow = Convert.ToInt32(e.CommandArgument);
                ((GridView)((ImageButton)e.CommandSource).Parent.Parent.Parent.Parent).SelectedIndex = intRow;
                //recuperem dni usuari
                usuDni = ((GridView)sender).SelectedDataKey[0].ToString();

                if (e.CommandName == "Eliminar")
                {
                    BLUsuari.EliminarUsuari(usuDni); //Eliminar usuari
                    gwGridConsultar.DataBind();
                }
                if (e.CommandName == "Editar")
                {
                    fvUsuario.DataSource = BLUsuari.GetEditarUsuari(usuDni); //Editar usuari
                    fvUsuario.DataBind();
                    fvUsuario.Visible = true;
                }
            }
            catch
            {
                MissatgesError.ErrorUsuari();
            }
        }
        #endregion


        #region[nou usuari]

        /// <summary>
        /// Cancel·lar alta nou usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnCancelar_Click(object sender, EventArgs e)
        {
            NetejarDades();
            mpeAltaUsuari.Hide();
        }

        /// <summary>
        /// Alta nou usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnEnviar_Click(object sender, EventArgs e)
        {
            try
            {
                //comprovem que estan els camps omplerts 
                if ((txtDniUsuari.Text.Length > 0 && txtNomNou.Text.Length > 0 && txtMail.Text.Length > 0 && txtClau.Text.Length > 0
                    && DropDownList1.SelectedValue != String.Empty && DropDownList2.SelectedValue != String.Empty))
                {
                    GuardarDades();
                    MissatgesError.DadesAfegides();
                    NetejarDades();
                    mpeAltaUsuari.Hide();
                }
                else
                {
                    string script = @"<script type='text/javascript'>alert('Falten camps per omplir (Telèfon,Pregunta,Resposta no són necessaris)');</script>";
                    ScriptManager.RegisterStartupScript(this, typeof(Page), "alerta", script, false);
                }
            }
            catch
            {
                MissatgesError.ErrorUsuari();
            }
        }

        /// <summary>
        /// Guardar dades usuari alta nova
        /// </summary>
        private void GuardarDades()
        {
            int departament = Int32.Parse(DropDownList1.SelectedValue);
            int rol = Int32.Parse(DropDownList2.SelectedValue);
            Usuari usu = new Usuari()
            {
                usudni = txtDniUsuari.Text,
                usucognom = txtNomNou.Text,
                email = txtMail.Text.Trim(),
                telefon = txtTelefon.Text,
                clau = txtClau.Text,
                pregunta = txtPregunta.Text,
                resposta = txtResposta.Text,
                idDepartament = departament,
                idRol = rol
            };
            //Crear nou usuari
            BLUsuari.AfegirUsuari(usu);
        }

        /// <summary>
        /// Netejar dades introduïdes 
        /// </summary>
        private void NetejarDades()
        {
            txtDniUsuari.Text = String.Empty;
            txtNomNou.Text = String.Empty;
            txtMail.Text = String.Empty;
            txtTelefon.Text = String.Empty;
            txtClau.Text = String.Empty;
            txtPregunta.Text = String.Empty;
            txtResposta.Text = String.Empty;
            DropDownList1.SelectedItem.Value = String.Empty;
            DropDownList2.SelectedItem.Value = String.Empty;
        }

        #endregion

        #region [editar usuari]
        /// <summary>
        /// Editar usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void EditarUsuari_Updating(object sender, FormViewUpdateEventArgs e)
        {
            try
            {
                int idusuari = Convert.ToInt32(((TextBox)fvUsuario.FindControl("txtIdUsuari")).Text);
                //Editem les dades de l'usuari seleccionat
                EditarDades(idusuari);
            }
            catch
            {
                MissatgesError.ErrorUsuari();
            }
        }

        protected void fvUsuario_OnModeChanging(object sender, FormViewModeEventArgs e)
        {
            fvUsuario.Visible = false;
        }

        /// <summary>
        /// Editar usuari seleccionat
        /// </summary>
        /// <param name="idUsuari">Id de l'usuari seleccionat</param>
        private void EditarDades(int idusuari)
        {
            Usuari usu = new Usuari()
            {
                idUsuari = idusuari,
                usudni = ((TextBox)fvUsuario.FindControl("txtDniEditar")).Text,
                usucognom = ((TextBox)fvUsuario.FindControl("txtNomEditar")).Text,
                email = ((TextBox)fvUsuario.FindControl("txtMailEditar")).Text.Trim(),
                telefon = ((TextBox)fvUsuario.FindControl("txtTelEditar")).Text,
                clau = ((TextBox)fvUsuario.FindControl("txtClauEditar")).Text,
                pregunta = ((TextBox)fvUsuario.FindControl("txtPreguntaEditar")).Text,
                resposta = ((TextBox)fvUsuario.FindControl("txtRespostaEditar")).Text,
                idDepartament = Int32.Parse(((DropDownList)fvUsuario.FindControl("DropDownList1")).SelectedValue),
                idRol = Int32.Parse(((DropDownList)fvUsuario.FindControl("DropDownList2")).SelectedValue)

            };
            BLUsuari.UpdateUsuari(usu); //Modificar usuari
            MissatgesError.Modificacio();
            fvUsuario.Visible = false;
        }

        /// <summary>
        /// Cancel·lar modificació usuari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param> 
        protected void UpdateCancelButton_Click(object sender, EventArgs e)
        {
            fvUsuario.Visible = false;
        }
        #endregion
    }
}