﻿using System;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Windows.Forms;
using Entities;
using Negoci;
using System.Web;
using GestioDespeses.Controls;

namespace GestioDespeses
{
    /// <summary>
    /// Gestió de les incidències obertes 
    /// </summary>
    public partial class Alertes : System.Web.UI.Page
    {
        DataTable dades = clsDataTable.TaulaPendents();
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                string usuari = String.Empty;

                if (HttpContext.Current.Session["usuari"] == null)
                    Response.Redirect("~/Account/Login.aspx");

                usuari = Convert.ToString(Session["usuari"]);
                string rolUsuari = BLRol.GetUsuariRol(usuari);
                if (rolUsuari != "Administrador")
                    Response.Redirect("~/Menu.aspx");    

                //Carregar grid amb totes les incidències pendents de respondre
                try
                {
                    CarregarIncidenciesObertes();
                }
                catch
                {
                    MissatgesError.ErrorCarrega();
                }
            }
        }

        #region[grids]

        protected void gwGridPendents_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                gwGridPendents.PageIndex = e.NewPageIndex;
                gwGridPendents.DataBind();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        /// <summary>
        /// Obtenim el id de la sol·licitud de la línia seleccionada
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gwGridPendents_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
        {
            //Selecció línia a editar
            int idincidencia = Convert.ToInt32(gwGridPendents.DataKeys[e.NewSelectedIndex].Value);
            //Guardar les dades de la incidència per recuperar-les al ModalPopUpExtender     
            CarregarDades(idincidencia);
            mpeEditResposta.Show();
        }

        protected void gwGridConsultar_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                gwGridConsultar.PageIndex = e.NewPageIndex;
                gwGridConsultar.DataBind();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        #endregion

        #region[metodes privats]

        /// <summary>
        /// Carregar incidències obertes
        /// </summary>
        private void CarregarIncidenciesObertes()
        {
            //Netejar la taula
            dades.Rows.Clear();
            dades.AcceptChanges();
            //llista incidències obertes
            var resultat = BLIncidencies.GetAll();

            foreach (var row in resultat)
            {
                //Cercar id usuari
                var usuari = BLUsuari.GetDni(row.IDUSUARI);
                //Afegir a la Taula les dades a visualitzar al grid
                dades.Rows.Add(row.IDINCIDENCIA, usuari.USUDNI, usuari.USUCOGNOM, row.INCBREUDESCRIPCIO, row.INCESTAT, row.INCALTA.ToShortDateString());
            }
            gwGridPendents.DataSource = dades;
            gwGridPendents.DataBind();
        }

        /// <summary>
        /// Carregar les dades obtingudes de la incidència seleccionada al grid
        /// </summary>
        /// <param name="idIncidencia">Id de la Incidència.</param>
        private void CarregarDades(int idIncidencia)
        {
            DBINCIDENCIES inc = BLIncidencies.GetIncidenciesId(idIncidencia);
            DBUSUARI usu = BLUsuari.GetDni(inc.IDUSUARI);
            lbInci.Text = Convert.ToString(inc.IDINCIDENCIA);
            lbDNIUsuari.Text = Convert.ToString(usu.USUDNI);
            lbNomUsuari.Text = Convert.ToString(usu.USUCOGNOM);
            txtEmailUsuari.Text = Convert.ToString(usu.USUEMAIL);
            lbBreuDes.Text = inc.INCBREUDESCRIPCIO;
            lbDataAltaInc.Text = Convert.ToString(inc.INCALTA.ToShortDateString());
            txtDetallDes.InnerText = inc.INCDESCRIPCIO;
        }

        /// <summary>
        /// Guardar les dades. Nova instància de clsIncidencies
        /// </summary>
        private void GuardarDades()
        {
            clsIncidencies inc = new clsIncidencies()
            {
                idIncidencia = Convert.ToInt32(lbInci.Text),
                idUsuari = BLUsuari.GetIdUsuari(lbDNIUsuari.Text),
                descripcio = txtDetallDes.InnerText,
                resposta = txtResposta.InnerText,
                breudescripcio = lbBreuDes.Text,
                dataAlta = Convert.ToDateTime(lbDataAltaInc.Text),
                nom = lbNomUsuari.Text
            };
            //S'actualitzen les dades amb la resposta i enviar email usuari
            BLIncidencies.ActualitzarIncidencia(inc);
        }

        #endregion

        #region[buttons]

        protected void btnCercar_Click(object sender, EventArgs e)
        {
            try
            {                
                gwGridConsultar.DataBind();
                gwConsultar.Visible = true;
            }
            catch
            {
                MissatgesError.DadesFiltreCorrectes();
            }
        }

        protected void btnTancar_Click(object sender, EventArgs e)
        {
            gwConsultar.Visible = false;
            gwGridConsultar.DataBind();
        }

        /// <summary>
        /// Enviar resposta a l'usuari i Tancar incidència
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnEnviar_Click(object sender, EventArgs e)
        {
            try
            {
                if (txtResposta.InnerText != String.Empty)
                {
                    GuardarDades();
                    txtResposta.InnerText = String.Empty;
                    //Actualitzar gwGridPendents
                    CarregarIncidenciesObertes();
                    mpeEditResposta.Hide();
                }
                else
                {
                    MissatgesError.CampsObligatoris();
                    mpeEditResposta.Show();
                }
            }
            catch
            {
                MissatgesError.ComprovarDades(); 
            }
        }

        protected void btnCancelar_Click(object sender, EventArgs e)
        {
            mpeEditResposta.Hide();
            txtResposta.InnerText = String.Empty;
        }

        /// <summary>
        /// Netejar dades del filtre
        /// </summary>
        protected void btnNetejar_Click(object sender, ImageClickEventArgs e)
        {
            txtIncidencia.Text = String.Empty;
            DropDownList1.SelectedIndex = 0;
            txtDataInici.Text = String.Empty;
            txtDataFi.Text = String.Empty;
        }
        #endregion
    }
}
