﻿using System;
using System.Web.UI.WebControls;
using System.Collections;
using System.Web;
using Negoci;
using GestioDespeses.Controls;

namespace GestioDespeses
{
    /// <summary>
    /// Autoritzar o Rebutjar sol·licituds
    /// </summary>
    public partial class Autoritzar : System.Web.UI.Page
    {
        static ArrayList ar = new ArrayList();
        private static string motiu = String.Empty;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                string usuari = String.Empty;

                if (HttpContext.Current.Session["usuari"] == null)
                    Response.Redirect("~/Account/Login.aspx");

                usuari = Convert.ToString(Session["usuari"]);
                string rolUsuari = BLRol.GetUsuariRol(usuari);
                if (rolUsuari != "Responsable")
                    Response.Redirect("~/Menu.aspx"); 

                 try
                 {
                     //Carreguem les solicituds en estat pendent
                     OmplirGrid();
                 }
                 catch
                 {
                     MissatgesError.ErrorCarrega();
                 }
            }
        }

        #region [grid]
        /// <summary>
        /// Carregar les dades obtingudes de la selecció al grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void CheckBox1_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cbi = (CheckBox)(sender);
            GridViewRow row = (GridViewRow)cbi.NamingContainer;

            if (cbi.Checked)
            {
                //si hi ha selecció, l'afegim al arraylist
                ar.Add(gwGridDetallAutoritzar.Rows[row.RowIndex].Cells[1].Text);
            }
            else
            {
                //si es desclica, l'eliminem de arraylist
                ar.Remove(gwGridDetallAutoritzar.Rows[row.RowIndex].Cells[1].Text);
            }
        }

        /// <summary>
        /// Mantenir els checkbox al canviar de pàgina al grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param> 
        protected void gwGridDetallAutoritzar_PageIndexChanged(object sender, EventArgs e)
        {
            OmplirGrid();
            //recuperem els checks seleccionats
            SeleccioGrid.RetornarSeleccionats((GridView)sender);  
        }

        /// <summary>
        /// Paginació
        /// </summary>
        protected void gwGridDetallAutoritzar_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                //Guardem els checks seleccionats
                SeleccioGrid.MantenirSeleccio((GridView)sender);
                gwGridDetallAutoritzar.PageIndex = e.NewPageIndex;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        #endregion

        #region[buttons]

        /// <summary>
        /// Autoritzar sol·licituds
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        protected void btnAutoritzar_Click(object sender, EventArgs e)
        {
            int idSol = 0;
            for (int c = 0; c < ar.Count; c++)
            {
                int solicitud = Convert.ToInt32(ar[c].ToString());
                //modifiquem l'estat de la sol·licitud a "Abonament"
                idSol = BLSolicitud.UpdateEstatSolicitud(solicitud);

                if (idSol > 0)
                {
                    Label1.Text = "*Autorització correcte";
                    NetejarCamps();
                }
                else
                {
                    Label1.Text = "*Autorització incorrecte";
                    NetejarCamps();
                }
            }
            //S'actualitza el grid
            OmplirGrid();
        }

        /// <summary>
        /// Rebutjar sol·licituds
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>  
        protected void btnRebutjar_Click(object sender, EventArgs e)
        {
            int idSol = 0;

            for (int c = 0; c < ar.Count; c++)
            {
                int solicitud = Convert.ToInt32(ar[c].ToString());
                //Modificar estat sol·licitud a "Rebutjada" i afegir motiu a la taula "DBAUTORITZACIO"
                idSol = BLSolicitud.UpdateEstatSolicitudRebutjar(solicitud, motiu);                

                if (idSol > 0)
                {
                    NetejarCamps();
                    Label2.Text = "*Rebuig correcte";
                }
                else
                {
                    NetejarCamps();
                    Label2.Text = "*Rebuig incorrecte";
                }
            }
            txtMotiu.InnerText = String.Empty;
            //Actualitzar grid
            OmplirGrid();
        }

        /// <summary>
        /// Consultar sol·licituds per DNI i/o data de la despesa
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnConsultar_Click(object sender, EventArgs e)
        {
            try
            {
                int sol = 0;
                btnTancar.Visible = true;
                NetejarCamps();
                DateTime? data = null;
                string dni = txtDNI.Text;
                string _data = txtData.Text;
                string registre = txtRegistre.Text;

                if (dni == String.Empty)
                {
                    gwGridDetallAutoritzar.DataBind();
                    Label3.Text = "*Cal informar un DNI";
                }
                else
                {
                    if (dni != String.Empty || _data != String.Empty || registre != String.Empty)
                    {
                        if (_data != String.Empty)
                            data = DateTime.Parse(_data);
                        //cerquem les sol·licituds segons els camps de la cerca 
                        gwGridDetallAutoritzar.DataSource = BLSolicitud.GetSolicitudsAutoritzarConsultar(data, dni, registre);
                        gwGridDetallAutoritzar.DataBind();
                    }
                }

                foreach (GridViewRow row in gwGridDetallAutoritzar.Rows)
                {
                    int _idsolicitud = Convert.ToInt32(gwGridDetallAutoritzar.Rows[row.RowIndex].Cells[3].Text);
                    //cerquem les sol·licituds en estat diferent a "Pendent"
                    sol = BLSolicitud.GetSolicitudIdPendent(_idsolicitud);

                    if (sol != 0)
                    {
                        //deshabilitem sol·licituds en estat diferent a "Pendent"
                        CheckBox chkbox2 = (CheckBox)row.FindControl("chkConsulta");
                        chkbox2.Enabled = false;
                    }
                }
            }
            catch
            {
                MissatgesError.DadesFiltreCorrectes();
            }
        }

        ///<summary>
        /// Afegir el motiu per rebutjar la sol·licitud
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnEnviar_Click(object sender, EventArgs e)
        {
            motiu = txtMotiu.InnerText;
            btnRebutjar_Click(sender, e);
        }

        protected void btnCancelar_Click(object sender, EventArgs e)
        {
            txtMotiu.InnerText = String.Empty;
        }

        ///<summary>
        /// Actualitzar grid només amb les sol·licituds pendents al tancar la consulta
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnTancar_Click(object sender, EventArgs e)
        {
            btnTancar.Visible = false;
            OmplirGrid();           
        }
        #endregion

        #region[métodes privats]
        ///<summary>
        /// Netejar etiquetes avís
        /// </summary>
        private void NetejarCamps()
        {
            Label1.Text = String.Empty;
            Label2.Text = String.Empty;
            Label3.Text = String.Empty;
        }

        ///<summary>
        /// Omplir grid amb les sol·licituds en estat pendent
        /// </summary>
        private void OmplirGrid()
        {
            //Carreguem les sol·licituds en estat pendent
            gwGridDetallAutoritzar.DataSource = BLSolicitud.GetSolicitudsPendentsAutoritzar();
            gwGridDetallAutoritzar.DataBind();
        }
        #endregion
    }
}
