﻿using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.Collections;
using System.Data;
using System.Web;
using Negoci;
using GestioDespeses.Controls;

namespace GestioDespeses
{
    /// <summary>
    /// Consultar les sol·licituds
    /// </summary>
    public partial class Consultar : System.Web.UI.Page
    {

        static ArrayList ar = new ArrayList();
        static List<DBSOLICITUD> llista = new List<DBSOLICITUD>();
        DataTable dades = clsDataTable.TaulaEditar();

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                if (HttpContext.Current.Session["usuari"] == null)
                    Response.Redirect("~/Account/Login.aspx");
                else
                {
                    string usuari = Session["usuari"].ToString();
                    //Consultem les dades de l'usuari 
                    var dades = BLUsuari.GetDadesUsuari(usuari);
                    foreach (DBUSUARI usu in dades)
                    {
                        lbDNIUsuari.Text = usu.USUDNI;
                        lbNomUsuari.Text = usu.USUCOGNOM;
                        lbEmailUsuari.Text = usu.USUEMAIL;
                    }
                }
            }
        }

        #region [grid]
        /// <summary>
        /// Carregar les dades obtingudes de la selecció al grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void CheckBox1_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cbi = (CheckBox)(sender);
            GridViewRow row = (GridViewRow)cbi.NamingContainer;

            if (cbi.Checked)
            {
                //si hi ha selecció, l'afegim al arraylist
                ar.Add(gwGridConsultar.Rows[row.RowIndex].Cells[1].Text);
            }
            else
            {
                //si es desclica, l'eliminem de arraylist
                ar.Remove(gwGridConsultar.Rows[row.RowIndex].Cells[1].Text);
            }
        }

        /// <summary>
        /// Mantenir les seleccions si es canvia de pàgina al grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gwGridConsultar_PageIndexChanged(object sender, EventArgs e)
        {
            OmplirGrid();
            SeleccioGrid.RetornarSeleccionats((GridView)sender);
        }

        /// <summary>
        /// Paginació grid consulta
        /// </summary>
        protected void gwGridConsultar_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                SeleccioGrid.MantenirSeleccio((GridView)sender);
                gwGridConsultar.PageIndex = e.NewPageIndex;
                OmplirGrid();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }


        /// <summary>
        /// Obtenim el id de la sol·licitud de la línia seleccionada
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gwGridDetall_SelectedIndexChanging(object sender, GridViewSelectEventArgs e)
        {

            int idsolicitud = Convert.ToInt32(gwGridConsultar.DataKeys[e.NewSelectedIndex].Value);
            //Redireccionem a la pàgina Menú amb id de la sol·licitud
            Response.Redirect(string.Format("~/Menu.aspx?id={0}", idsolicitud));
        }

        protected void gwGridDetall_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                gwGridDetall.PageIndex = e.NewPageIndex;
                gwGridDetall.DataBind();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        #endregion

        #region [metodes privats]
        /// <summary>
        /// Omplir dades el grid segons filtre
        /// </summary>
        private void OmplirGrid()
        {
            gwGridConsultar.DataSource = llista;
            gwGridConsultar.DataBind();
        }      
         #endregion

        #region [buttons]

        /// <summary>
        /// Botó Consultar sol·licituds
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnConsultar_Click(object sender, EventArgs e)
        {
            try
            {
                string _solicitud = txtSolicitud.Text;
                string _data = txtData.Text;
                string _estat = ddlEstat.SelectedValue;
                string dni = lbDNIUsuari.Text;
                gwConsultar.Visible = true;
                //si algún dels camps de la cerca està omplert
                if (_solicitud != String.Empty || _data != String.Empty || _estat != String.Empty)
                {
                    DateTime? data = null;
                    if (_data != String.Empty)
                    {
                        data = Convert.ToDateTime(_data);
                    }

                    llista = BLSolicitud.GetAllSolicituds(dni, _solicitud, data, _estat);
                    OmplirGrid();
                    btnGridEliminar.Enabled = true;
                }
                else
                {
                    llista = BLSolicitud.GetAllSolicitudsSenseFiltre(dni);
                    OmplirGrid();
                }

                int sol = 0;
                foreach (GridViewRow row in gwGridConsultar.Rows)
                {
                    int _idsolicitud = Convert.ToInt32(gwGridConsultar.Rows[row.RowIndex].Cells[1].Text);
                    //obtenim les sol·licituds que no estiguin ni rebutjades ni obertes              
                    sol = BLSolicitud.GetSolicitudIdNoOberta(_idsolicitud);

                    if (sol != 0)
                    {
                        //deseleccionem les sol·licituds que no estan rebutjades ni obertes
                        CheckBox chkbox2 = (CheckBox)row.FindControl("chkConsulta");
                        chkbox2.Enabled = false;
                    }
                }
            }
            catch
            {
                MissatgesError.DadesFiltreCorrectes();
            }
        }      

        /// <summary>
        /// Botó Editar sol·licituds
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnEditar_Click(object sender, EventArgs e)
        {
            divGridDetall.Visible = true;
            dades.Rows.Clear();
            dades.AcceptChanges();
            for (int c = 0; c < ar.Count; c++)
            {
                int solicitud = Convert.ToInt32(ar[c].ToString());
                //Obtenim les dades de les sol·licituds seleccionades al gwGridConsultar
                var resultat = BLSolicitud.GetSolicitudsEditar(solicitud);

                foreach (var fila in resultat)
                {
                    dades.Rows.Add(fila.IDSOLICITUD, fila.SOLIMPORT, Convert.ToDateTime(fila.SOLDATADESPESA).ToShortDateString(), fila.SOLDESCRIPCIO, fila.SOLARXIU);
                }
            }
            //omplim el grid amb les sol·licituds seleccionades al gwGridConsultar
            gwGridDetall.DataSource = dades;
            gwGridDetall.DataBind();
        }

        /// <summary>
        /// Botó Eliminar sol·licituds 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnDelete_Click(object sender, EventArgs e)
        {
            try
            {
                int resul = 0;
                for (int c = 0; c < ar.Count; c++)
                {
                    int solicitud = Convert.ToInt32(ar[c].ToString());
                    resul = solicitud;
                    //Eliminem les sol·licituds seleccionades
                    BLSolicitud.EliminarSolicitud(solicitud);
                    ar.Remove(ar[c].ToString());

                    if (resul == 1)
                    {
                        c = -1;
                    }
                }
                OmplirGrid();
                btnEditar_Click(sender, e);
            }
            catch
            {
                MissatgesError.EliminarRegistre();
            }
        }

        /// <summary>
        /// Botó Tancar gwGridDetall
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnTancar_Click(object sender, EventArgs e)
        {
            divGridDetall.Visible = false;
            divGridDetall.DataBind();
        }

        /// <summary>
        /// Botó Netejar filtre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnNetejar_Click(object sender, System.Web.UI.ImageClickEventArgs e)
        {
            txtSolicitud.Text = String.Empty;
            txtData.Text = String.Empty;
            ddlEstat.SelectedValue = String.Empty;
        }
        #endregion
    }
}
