﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;

namespace GestioDespeses.Controls
{
    public static class SeleccioGrid
    {
        static List<int> clicats = new List<int>();

        /// <summary>
        /// Guardem els registres seleccionats per recuperar-los posteriorment
        /// </summary>
        /// <param name="grid"></param>
        public static void MantenirSeleccio(GridView grid)
        {
            //Busquem els registres seleccionats i els afegim a una llista
            foreach (GridViewRow row in grid.Rows.Cast<GridViewRow>())
            {
                CheckBox chkbox1 = (CheckBox)row.FindControl("chkConsulta");
                if (chkbox1.Checked)
                {
                    clicats.Add(Convert.ToInt32(row.Cells[1].Text));
                }
            }

            //Es recupera la llista dels marcats anteriorment
            List<int> anteriors = HttpContext.Current.Session["seleccionats"] as List<int>;

            if (anteriors == null)
                anteriors = new List<int>();
            //es comparen els registres de la pàgina actual amb les guardats per no duplicar
            anteriors = (from item in anteriors
                         join item2 in grid.Rows.Cast<GridViewRow>()
                         on item equals Convert.ToInt32(grid.DataKeys[item2.RowIndex].Value) into valor
                         where !valor.Any()
                         select item).ToList();          

            anteriors.AddRange(clicats);
            HttpContext.Current.Session["seleccionats"] = anteriors;
        }
        
        /// <summary>
        /// Tornar a seleccionar els cheks marcats quan canviem de pàgina en el grid
        /// </summary>
        /// <param name="grid"></param>
        public static void RetornarSeleccionats(GridView grid)
        {
            //recuperem valors de la sessió
            List<int> marcats = HttpContext.Current.Session["seleccionats"] as List<int>;

            if (marcats == null)
                return;

            for (int c = 0; c < marcats.Count; c++)
            {
                foreach (GridViewRow row in grid.Rows.Cast<GridViewRow>())
                {
                    //comparem i si estan en la pàgina actual els marquem com seleccionats
                    if (marcats[c].ToString() == row.Cells[1].Text)
                    {
                        CheckBox chkbox1 = (CheckBox)row.FindControl("chkConsulta");
                        chkbox1.Checked = true;
                    }
                }
            }
        }
    }
}
