﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Negoci;
using System.Web.UI;
using GestioDespeses.Controls;


namespace GestioDespeses
{
    /// <summary>
    /// Històric de sol·licituds
    /// </summary>
    public partial class Historic : System.Web.UI.Page
    {
        DataTable filtered = clsDataTable.TaulaHistoric();
        static List<DBSOLICITUD> llista = new List<DBSOLICITUD>();
        DataTable dades = new DataTable();

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                string usuari = String.Empty;

                if (Session["usuari"] == null)
                    Response.Redirect("~/Account/Login.aspx");

                usuari = Convert.ToString(Session["usuari"]);
                string rolUsuari = BLRol.GetUsuariRol(usuari);
                if (rolUsuari != "Responsable")
                    Response.Redirect("~/Menu.aspx"); 

                btnCercar.Enabled = true;
            }
        }

        /// <summary>
        /// Cercar sol·licituds de la sel·leció gwGridHistoric
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void gwGridHistoric_SelectedIndexChanging(object sender, EventArgs e)
        {
            divGridDetall.Visible = true;
            string _tipologia = ddlTipologia.SelectedValue;
            dades = clsDataTable.TaulaDetallHistoric();
            for (int c = 0; c < llista.Count; c++)
            {
                int solicitud = Convert.ToInt32(llista.ElementAt(c).IDSOLICITUD);
                //recuperem totes les sol·licituds de la selecció
                var resultat = BLConsultaHistoric.GetSolicitudsHistoric(solicitud);
                //Dades usuari sol·licitud
                var usuari = BLUsuari.GetDniUsuari(solicitud);

                foreach (var fila in usuari)
                {
                    foreach (var row in resultat)
                    {
                        //Get descripció tipologia
                        string tipus = BLTipologia.GetDescripcioTipologia(row.IDTIPOLOGIA);

                        dades.Rows.Add(fila.USUDNI, fila.USUCOGNOM, row.IDSOLICITUD, row.SOLESTAT, Convert.ToDateTime(row.SOLDATADESPESA).ToShortDateString(),
                          tipus, row.SOLDESCRIPCIO, row.SOLIMPORT, row.SOLNUMREGISTRE,
                                 Convert.ToDateTime(row.SOLDATAALTA).ToShortDateString(), Convert.ToDateTime(row.SOLDATAFI).ToShortDateString());
                    }
                }
            }
            //dades grid per recuperar al Excel
            Session["gwGridDetallHistoric"] = dades;
            //Omplim gwGridDetallHistoric
            gwGridDetallHistoric.DataSource = dades;
            gwGridDetallHistoric.DataBind();
        }

        #region[button]

        /// <summary>
        /// Cercar sol·licituds segons filtre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnCercar_Click(object sender, EventArgs e)
        {
            try
            {
                gwHistoric.Visible = true;
                string _tipologia = ddlTipologia.SelectedValue;
                string oDataInici = txtDataInici.Text;
                string oDataFi = txtDataFi.Text;
                int numero = 0;
                float import = 0;
                llista.Clear();

                //si algún dels camps de la cerca està omplert
                if (_tipologia != String.Empty || oDataInici != String.Empty || oDataFi != String.Empty)
                {
                    DateTime? dataInici = null;
                    DateTime? dataFi = null;

                    if (oDataInici != String.Empty || oDataFi != String.Empty)
                    {
                        dataInici = Convert.ToDateTime(oDataInici);
                        dataFi = Convert.ToDateTime(oDataFi);
                    }
                    //recuperem el número total de sol·licituds d'un departament segons cerca 
                    numero = BLConsultaHistoric.GetHistoricQuantitat(_tipologia, dataInici, dataFi);
                    //recuperem import total de les sol·licituds d'un departament
                    import = BLConsultaHistoric.GetHistoricImport(_tipologia, dataInici, dataFi);
                    //afegim totes les sol·licituds resultants per recuperar-las en la selecció detall
                    llista = BLConsultaHistoric.GetHistoricLlista(_tipologia, dataInici, dataFi);

                    filtered.Rows.Add(numero, _tipologia, import, dataInici, dataFi);
                }
                //mostrem les dades gwGridHistoric
                gwGridHistoric.DataSource = filtered;
                gwGridHistoric.DataBind();
            }
            catch
            {
                MissatgesError.DadesFiltreCorrectes();
            }

        }

        /// <summary>
        /// Redireccionar Extreure històric sol·licituds en format Excel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ImageButton1_Click(object sender, ImageClickEventArgs e)
        {
            Response.Redirect("~/GuardarExcel.aspx");
        }

        /// <summary>
        /// Tancar grid detall consulta sol·licituds
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnTancar_Click(object sender, EventArgs e)
        {
            divGridDetall.Visible = false;
            gwGridDetallHistoric.DataBind();
        }        

        /// <summary>
        /// Netejar dades filtre
        /// </summary>
        protected void btnNetejar_Click(object sender, ImageClickEventArgs e)
        {
            ddlTipologia.SelectedIndex = 0;
            txtDataInici.Text = String.Empty;
            txtDataFi.Text = String.Empty;
        }
        #endregion
    }
}
