﻿using System;
using System.Web;
using System.IO;
using Negoci;
using System.Windows.Forms;
using Entities;
using System.Web.UI;
using GestioDespeses.Controls;

namespace GestioDespeses
{
    /// <summary>
    /// Afegir nova despesa
    /// </summary>
    public partial class Menu : System.Web.UI.Page
    {

        private int IdSolicitud = 0;
        private string filename = String.Empty;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                if (HttpContext.Current.Session["usuari"] == null)
                {
                    Response.Redirect("~/Account/Login.aspx");
                }
                else
                {
                    string usuari = Session["usuari"].ToString();
                    //Obtenim dades de l'usuari registrat
                    var dades = BLUsuari.GetDadesUsuari(usuari);
                    foreach (var usu in dades)
                    {
                        lbDNIUsuari.Text = usu.USUDNI;
                        lbNomUsuari.Text = usu.USUCOGNOM;
                        lbEmailUsuari.Text = usu.USUEMAIL;
                    }
                }
            }
            //si la sol·licitud és modifica
            if (Request.Params["id"] != null)
            {
                this.IdSolicitud = Convert.ToInt32(Request.Params["id"]);
                //Carreguem les dades de la sol·licitud a modificar
                CarregarDades(IdSolicitud);
            }
        }

        #region[button]
        /// <summary>
        /// Enviar sol·licitud a autoritzar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnAutoritzar_Click1(object sender, EventArgs e)
        {
            try
            {
                string script = @"<script type='text/javascript'>alert('Falten camps per omplir (Arxiu no és obligatori)');</script>";
                if (ComprovarValidacions() == true)
                {
                    //Guardem les dades
                    int solicitud = GuardarDades();
                    //Modifiquem la sol·licitud a l'estat "Pendent"
                    string registre = BLSolicitud.UpdateEstatSolicitudAutoritzar(solicitud);
                    string msg = string.Format("alert('Sol·licitud {0} enviada correctament amb el número de registre {1}')", Convert.ToString(solicitud), registre);
                    ScriptManager.RegisterStartupScript(this, typeof(Page), "Info", msg, true);
                    NetejarDades();
                }
                else
                    ScriptManager.RegisterStartupScript(this, typeof(Page), "alerta", script, false);
            }
            catch
            {
                MissatgesError.ComprovarDades();
            }
        }

        /// <summary>
        /// Guardar sol·licitud pendent autoritzar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnGuardar_Click(object sender, EventArgs e)
        {
            try
            {   
                //Si els camps obligatoris estan omplerts guardem   
                if (ComprovarValidacions() == true)
                {
                    int numero = GuardarDades();
                    //Comprovem si existeix per modificar          
                    bool existeix = BLSolicitud.Existeix(IdSolicitud);
                    if (existeix == true)
                    {
                        MissatgesError.Actualitzat();
                    }
                    else
                    {
                        string script = string.Format("alert('Sol·licitud creada amb el número {0}')", Convert.ToString(numero));
                        ScriptManager.RegisterStartupScript(this, typeof(Page), "Info", script, true);
                        NetejarDades();
                    }
                }
                else
                {
                    string script = @"alert('Falten camps per omplir (Arxiu no és obligatori)');";
                    ScriptManager.RegisterStartupScript(this, typeof(Page), "alerta", script, true);
                }
            }
            catch
            {
                MissatgesError.DadesNoAfegides();
            }
        }

        /// <summary>
        /// Cancel·lar 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnCancelar_Click(object sender, EventArgs e)
        {
            if (Request.Params["id"] != null)
            {
                //si era la modificació d'una sol·licitud retornem a les consultes
                Response.Redirect(string.Format("~/Consultar.aspx"));
            }
            else
            {
                NetejarDades();
            }
        }
        #endregion

        #region[métodes privats]
        /// <summary>
        /// Guardar les dades de la nova sol·licitud
        /// </summary>
        /// <returns>Retorna les dades introduïdes per l'usuari</returns>
        private Solicitud DadesSolicituds()
        {
            int idUsu = BLUsuari.GetIdUsuari(lbDNIUsuari.Text);
            DateTime data = Convert.ToDateTime(txtData.Text);
            string tipus = DropDownList1.SelectedValue;
            int _tipologia = BLTipologia.GetIdTipologia(tipus);
            int calendari = BLCalendari.GetIdCalendari(data);
            Solicitud sol = new Solicitud()
                     {
                         idSolicitud = IdSolicitud,
                         idUsuari = idUsu,
                         idTipologia = _tipologia,
                         idCalendari = calendari,
                         descripcio = txtDescipcio.Text,
                         import = Convert.ToSingle(txtImport.Text),
                         dataDespesa = Convert.ToDateTime(txtData.Text),
                         arxiu = HiHaArxiu()
                     };
            return sol;
        }

        /// <summary>
        /// Donar d'alta una sol·licitud 
        /// </summary>
        /// <returns>Retorna id sol·licitud nova o modificada</returns>
        private int GuardarDades()
        {
            int idSol = 0;
            string imatgeTmpName = string.Empty;
            string email = lbEmailUsuari.Text.Trim();
            string arxiu = String.Empty;
            string nom = lbNomUsuari.Text.Trim();
            //Comprovem si existeix per modificar          
            bool existeix = BLSolicitud.Existeix(IdSolicitud);

            if (existeix == true)
            {
                Solicitud sol = DadesSolicituds();
                //modifiquem les dades
                BLSolicitud.UpdateSolicitud(sol);
                if (FileUpload1.HasFile)
                {
                    bool existArxiu = BLArxiu.ExisteixArxiu(IdSolicitud);
                    if (existArxiu == true)
                    {
                        //Modifiquem la taula arxius
                        ActualitzarArxiu(IdSolicitud);
                    }
                    else
                    {
                        //alta nou arxiu amb idSolicitud a modificar
                        guardarArxiu(IdSolicitud);
                    }
                }
                 idSol = IdSolicitud;
            }
            else
            {
                Solicitud sol = DadesSolicituds();
                arxiu = HiHaArxiu();
                //Alta nova sol·licitud
                var solicitud = BLSolicitud.AfegirSolicitud(sol, nom, email);

                if (solicitud > 0)
                {
                    //si té arxiu guardem amb el id de la sol·licitud
                    if (FileUpload1.HasFile)
                    {
                        guardarArxiu(solicitud);
                    }                     
                     NetejarDades();
                }
                else
                {
                    MissatgesError.DadesNoAfegides();
                }
                idSol = solicitud;
            }
            return idSol;
        }

        /// <summary>
        /// Donar format arxiu per guardar en bytes
        /// </summary>
        /// <returns>Retorna byte arxiu</returns>
        private byte[] FormatArxiu()
        {
            filename = Path.GetFileName(FileUpload1.PostedFile.FileName);
            string contenttype = Path.GetFileName(FileUpload1.PostedFile.ContentType);
            string ext = Path.GetExtension(FileUpload1.PostedFile.FileName);
            Stream stream = FileUpload1.PostedFile.InputStream;
            byte[] buffer = new byte[8000];
            stream.Read(buffer, 0, 8000);
            return buffer;
        }

        /// <summary>
        /// Donar format arxiu per guardar en bytes
        /// </summary>
        /// <param name="solicitud">Id de la sol·licitud.</param>
        private void guardarArxiu(int solicitud)
        {
            byte[] buffer = FormatArxiu();
            //guardar arxiu en bytes amb el número de la sol·licitud
            BLArxiu.AfegirArxiu(solicitud, filename, FileUpload1.PostedFile.ContentLength, buffer);
        }

        /// <summary>
        /// Modificar arxiu
        /// </summary>
        /// <param name="solicitud">Id de la sol·licitud.</param>
        private void ActualitzarArxiu(int solicitud)
        {
            byte[] buffer = FormatArxiu();
            BLArxiu.ActualitzarArxiu(solicitud, filename, FileUpload1.PostedFile.ContentLength, buffer);
        }

        /// <summary>
        /// Comprova si hi ha arxiu adjuntat
        /// </summary>
        /// <returns>Retorna bool</returns>
        private string HiHaArxiu()
        {
            string arxiu = String.Empty;
            if (FileUpload1.HasFile)
            {
                arxiu = "S";
            }
            else { arxiu = "N"; }
            return arxiu;
        }

        /// <summary>
        /// Carregar dades
        /// </summary>
        /// <param name="id">Id de la sol·licitud.</param>
        private void CarregarDades(int id)
        {
            //Recuperem les dades de la sol·licitud
            var solicitud = BLSolicitud.GetSolicitudId(id);
            string tipologia = BLTipologia.GetDescripcioTipologia(solicitud.IDTIPOLOGIA);
            var arx = BLArxiu.GetByArxiu(id);
            txtImport.Text = Convert.ToString(solicitud.SOLIMPORT);
            txtData.Text = Convert.ToString(solicitud.SOLDATAALTA.Value.ToShortDateString());
            DropDownList1.SelectedValue = tipologia;
            txtDescipcio.Text = Convert.ToString(solicitud.SOLDESCRIPCIO);            
        }

        /// <summary>
        /// Netejar els camps de les dades
        /// </summary>
        private void NetejarDades()
        {
            txtImport.Text = String.Empty;
            txtData.Text = String.Empty;
            DropDownList1.SelectedValue = String.Empty;
            txtDescipcio.Text = String.Empty;
            FileUpload1.Dispose();
        }

        /// <summary>
        /// Comprovar que els camps obligatoris estiguin omplerts
        /// </summary>
        /// <returns>True si els camps estan omplerts sino False</returns>
        private bool ComprovarValidacions()
        {
            if (txtImport.Text != String.Empty && txtData.Text != String.Empty && DropDownList1.SelectedValue != String.Empty
                && txtDescipcio.Text != String.Empty)
                return true;
            else
                return false;
        }

        #endregion
    }
}