﻿using System;
using System.Collections.Generic;
using System.Data;
using Dades;
using Entities;
using GestioDespeses;



namespace Negoci
{
    /// <summary>
    /// Sol·licitud de la despesa
    /// </summary>
    public class BLSolicitud
    {
        //Llista totes les sol·licituds per editar
        public static List<DBSOLICITUD> GetSolicitudsEditar(int solicitud)
        {
            return new DASolicitud().GetSolicitudsEditar(solicitud);
        }
        //Retorna id sol·licituds no obertes o rebutjades
        public static int GetSolicitudIdNoOberta(int solicitud)
        {
            return new DASolicitud().GetSolicitudIdNoOberta(solicitud);
        }
        //Llista sol·licituds en estat pendent
        public static int GetSolicitudIdPendent(int solicitud)
        {
            return new DASolicitud().GetSolicitudIdPendent(solicitud);
        }
        //Retorna dades sol·licitud 
        public static DBSOLICITUD GetSolicitudId(int? solicitud)
        {
            return new DASolicitud().GetSolicitudId(solicitud);
        }
        //Alta nova sol·licitud
        public static int AfegirSolicitud(Solicitud sol, string nom, string email)
        {
            return new DASolicitud().AfegirSolicitud(sol, nom, email);
        }
        //Llista sol·licituds segons tipologia
        public static List<DBSOLICITUD> GetSolicituds(int tipologia)
        {
            return new DASolicitud().GetSolicitudsEditar(tipologia);
        }
        //Llista sol·licituds segons parametres 
        public static List<DBSOLICITUD>GetAllSolicituds(string dni,string _solicitud, DateTime? data,string _estat)
        {
            int usuari = BLUsuari.GetIdUsuari(dni);
            return new DASolicitud().GetAllSolicituds(usuari, _solicitud, data, _estat);
        }

        //Llista solicituds sense filtres
        public static List<DBSOLICITUD> GetAllSolicitudsSenseFiltre(string usuari)
        {
            int _usuari = BLUsuari.GetIdUsuari(usuari);
            return new DASolicitud().GetAllSolicitudsSenseFiltre(_usuari);
        }

        //Sol·licituds pendents d'autoritzar
        public static DataTable GetSolicitudsPendentsAutoritzar()
        {
            DASolicitud llista = new DASolicitud();
            DataTable dades = clsDataTable.TaulaPendentAutoritzar();
            var resultat = llista.GetSolicitudsPendentsAutoritzar();

            foreach (var row in resultat)
            {
                var usuari = BLUsuari.GetDniUsuari(row.IDSOLICITUD);
                string tip = BLTipologia.GetDescripcioTipologia(row.IDTIPOLOGIA);
                string calendari = BLCalendari.GetDescripcioCalendari(Convert.ToDateTime(row.SOLDATADESPESA));
                foreach (var fila in usuari)
                {
                    dades.Rows.Add(fila.USUDNI, fila.USUCOGNOM, row.IDSOLICITUD, row.SOLESTAT, calendari, Convert.ToDateTime(row.SOLDATADESPESA).ToShortDateString(),
                        tip, row.SOLDESCRIPCIO, row.SOLIMPORT, row.SOLNUMREGISTRE,
                               Convert.ToDateTime(row.SOLDATAALTA).ToShortDateString());
                }
            }
            return dades;
        }
        //Actualitza sol·licituds. Canvia estat a Abonament
        public static int UpdateEstatSolicitud(int solicitud)
        {
            return new DASolicitud().UpdateEstatSolicitud(solicitud);
        }
        //Actualitza sol·licituds 
        public static void UpdateSolicitud(Solicitud solicitud)
        {
            DASolicitud sol = new DASolicitud();
            sol.UpdateSolicitud(solicitud);
        }
        //Retorna id sol·licituds rebutjades. Canvi estat a rebutjat.
        public static int UpdateEstatSolicitudRebutjar(int solicitud, string motiu)
        {
            //Afegir el motiu
            BLAutoritzacio.AfegirRebuig(solicitud, motiu);
            return new DASolicitud().UpdateEstatSolicitudRebutjar(solicitud, motiu);
        }
        //Retorna id sol·licituds autoritzades. Canvi estat a Abonament
        public static string UpdateEstatSolicitudAutoritzar(int solicitud)
        {
            return new DASolicitud().UpdateEstatSolicitudAutoritzar(solicitud);
        }
        //llista totes les sol·licituds segons paràmetres d'entrada en la cerca pàgina Autoritzar
        public static DataTable GetSolicitudsAutoritzarConsultar(DateTime? data, string dni, string registre)
        {
            DASolicitud llista = new DASolicitud();
            DataTable dades = clsDataTable.TaulaPendentAutoritzar();
            var resultat = llista.GetSolicitudsAutoritzarConsultar(data, dni, registre);

            foreach (var row in resultat)
            {
                var usuari = BLUsuari.GetDniUsuari(row.IDSOLICITUD);
                foreach (var fila in usuari)
                {
                    dades.Rows.Add(fila.USUDNI, fila.USUCOGNOM, row.IDSOLICITUD, row.SOLESTAT, Convert.ToDateTime(row.SOLDATADESPESA).ToShortDateString(),
                        row.IDTIPOLOGIA, row.SOLDESCRIPCIO, row.SOLIMPORT, row.SOLNUMREGISTRE,
                               Convert.ToDateTime(row.SOLDATAALTA).ToShortDateString());
                }
            }
            return dades;
        }
        //Elimina sol·licituds
        public static void EliminarSolicitud(int solicitud)
        {
            DASolicitud del = new DASolicitud();
            del.EliminarSolicitud(solicitud);
        }
        //Comprova si existeix sol·licitud
        public static bool Existeix(int idSolicitud)
        {
            return new DASolicitud().Existeix(idSolicitud);
        }
       
    }
}
