﻿using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public class GridViewExportUtil
{
    /// <summary>
    /// Per Exportar el Grid a Excel
    /// </summary>
    /// <param name="fileName"></param>
    /// <param name="gv"></param>
    public static void Export(string fileName, GridView grid)
    {
        HttpContext.Current.Response.Clear();
        HttpContext.Current.Response.AddHeader("content-disposition", string.Format("attachment; filename={0}", fileName));
        HttpContext.Current.Response.ContentType = "application/ms-excel";

        using (StringWriter sw = new StringWriter())
        {
            using (HtmlTextWriter htw = new HtmlTextWriter(sw))
            {
                Table taula = new Table();

                //afegim el grid a la taula
                taula.GridLines = grid.GridLines;

                //Capçalera
                if (grid.HeaderRow != null)
                {
                    GridViewExportUtil.PrepararControls(grid.HeaderRow);
                    taula.Rows.Add(grid.HeaderRow);
                }
                foreach (GridViewRow row in grid.Rows)
                {
                    //Prepara el grid substituint els controladors
                    GridViewExportUtil.PrepararControls(row);
                    taula.Rows.Add(row);
                }
                //Peu 
                if (grid.FooterRow != null)
                {
                    GridViewExportUtil.PrepararControls(grid.FooterRow);
                    taula.Rows.Add(grid.FooterRow);
                }
                //Render la taula en htmlwriter
                taula.RenderControl(htw);

                //Render de htmlwriter
                HttpContext.Current.Response.Write(sw.ToString());
                HttpContext.Current.Response.End();
            }
        }
    }

    /// <summary>
    /// Substitueix controladors que estiguin dintre el grid
    /// </summary>
    /// <param name="control"></param>
    private static void PrepararControls(Control control)
    {
        for (int i = 0; i < control.Controls.Count; i++)
        {
            Control actual = control.Controls[i];
            if (actual is LinkButton)
            {
                control.Controls.Remove(actual);
                control.Controls.AddAt(i, new LiteralControl((actual as LinkButton).Text));
            }
            else if (actual is ImageButton)
            {
                control.Controls.Remove(actual);
                control.Controls.AddAt(i, new LiteralControl((actual as ImageButton).AlternateText));
            }
            else if (actual is HyperLink)
            {
                control.Controls.Remove(actual);
                control.Controls.AddAt(i, new LiteralControl((actual as HyperLink).Text));
            }
            else if (actual is DropDownList)
            {
                control.Controls.Remove(actual);
                control.Controls.AddAt(i, new LiteralControl((actual as DropDownList).SelectedItem.Text));
            }
            else if (actual is CheckBox)
            {
                control.Controls.Remove(actual);
                control.Controls.AddAt(i, new LiteralControl((actual as CheckBox).Checked ? "True" : "False"));
            }

            if (actual.HasControls())
            {
                GridViewExportUtil.PrepararControls(actual);
            }
        }
    }
}

